import Gempyre
import os
import sys
from pathlib import Path
from Gempyre import resource

def main():
    name = os.path.join(os.path.dirname(sys.argv[0]), "python_dialog_test.html")
    map, names = resource.from_file(name)
    ui = Gempyre.Ui(map, names[name])
    output = Gempyre.Element(ui, "output")
    open_file_button = Gempyre.Element(ui, "open_file")
    def open_file(_):
        dlg = Gempyre.Dialog()
        result = dlg.open_file_dialog("hop", str(Path.home()), [("Text", ["*.txt", "*.text"])])
        if result:
            output.set_html(result)
        
    open_file_button.subscribe("click", open_file)
    
    open_files_button = Gempyre.Element(ui, "open_files")
    def open_files(_):
        result = Gempyre.Dialog().open_files_dialog("hop", str(Path.home()))
        if result:
            output.set_html(str(result))
        
    open_files_button.subscribe("click", open_files)
    
    open_dir_button = Gempyre.Element(ui, "open_dir")
    def open_dir(_):
        dialog = Gempyre.Dialog()
        result = dialog.open_dir_dialog("hop", str(Path.home()))
        if result:
            output.set_html(result)
        
    open_dir_button.subscribe("click", open_dir)
    
    
    save_file_button = Gempyre.Element(ui, "save_file")
    def save_file(_):
        dialog = Gempyre.Dialog()
        filter = [("Foo", ["*.foo", "*.bar"])]
        result = dialog.save_file_dialog("hop", str(Path.home()), filter)
        if result:
            output.set_html(result)
        
    save_file_button.subscribe("click", save_file)
    
    ui.run()

if __name__ == "__main__":
    main()
    



import sys
import os
import math
import random
from datetime import timedelta
import Gempyre
from Gempyre import resource

TICK_SPEED = 30
MONSTER_SPEED = 0.15
BULLET_SPEED = 1.0
GUN_SPEED = 5.0
NUMBERS_WIDTH = 13
NUMBERS_HEIGHT = 25
BULLET_WIDTH = 20
BULLET_HEIGHT = 20
GUN_HEIGHT = 50
GUN_WIDTH = 100
BARREL_WIDTH = 10
BARREL_HEIGHT = 40
MONSTER_WIDTH = 40
MONSTER_HEIGHT = 40
TURRET_STEP = 0.01
MAX_AMMO = 100
GAME_SPEED = 50


class Number:
    def __init__(self, images):
        self.images = images

    def draw(self, g, x, y, width, height, value):
        g.draw_image_clip(self.images, Gempyre.Rect(
            NUMBERS_WIDTH * value, 0, NUMBERS_WIDTH, NUMBERS_HEIGHT), Gempyre.Rect(x, y, width, height))


class Monster:
    def __init__(self, x, y, width, height, endurance, numbers):
        self.step_y = MONSTER_SPEED
        self.step_x = 0
        self.x = x
        self.y = y
        self.width = width
        self.height = height
        self.numbers = numbers
        self.endurance = endurance

    def step(self):
        self.y += self.step_y

    def test_inside(self, x, y, width, height):
        return self.y + self.height > y + height

    def draw(self, g, image):
        g.draw_image_rect(image, Gempyre.Rect(self.x, self.y, self.width, self.height))
        f1 = int(self.endurance / 10)
        f2 = self.endurance - (f1 * 10)
        self.numbers.draw(g, self.x + 6, self.y + 6, 10, 10, f1)
        self.numbers.draw(g, self.x + 24, self.y + 6, 10, 10, f2)


class Bullet:
    def __init__(self, direction, x, y):
        self.speed = BULLET_SPEED
        self.x = x
        self.y = y
        self.step_x = math.sin(direction) * self.speed
        self.step_y = math.cos(direction) * self.speed
        self.hit_in = None
        self.LEFT = 1
        self.TOP = 2
        self.RIGHT = 4
        self.BOTTOM = 8

    def step(self):
        self.x += self.step_x
        self.y += self.step_y

    def test_inside(self, x, y, width, height):
        if self.x < x:
            self.step_x *= -1.0
            self.x = x
        elif self.x + BULLET_WIDTH > x + width:
            self.step_x *= -1.0
            self.x = width - BULLET_WIDTH
        elif self.y < y:
            self.step_y *= -1.0
            self.y = y
        elif self.y + BULLET_HEIGHT > y + height:
            self.step_y *= -1.0
            self.y = height - BULLET_HEIGHT
            return True
        return False

    def is_inside(self, other):
        d_dir = 0
        if self.x <= other.x:
            d_dir |= self.LEFT
        if self.x + BULLET_WIDTH >= other.x + other.width:
            d_dir |= self.RIGHT
        if self.y <= other.y:
            d_dir |= self.TOP
        if self.y + BULLET_HEIGHT >= other.y + other.height:
            d_dir |= self.BOTTOM
        return d_dir

    def test_hit(self, other):
        ox = other.x + other.width
        oy = other.y + other.height
        sx = self.x + BULLET_WIDTH
        sy = self.y + BULLET_HEIGHT
        if self.x < ox and sx > other.x and self.y < oy and sy > other.y:
            if self.hit_in:
                return False
            l = self.x - other.x
            r = (self.x + BULLET_WIDTH) - (other.x + other.width)
            t = self.y - other.y
            b = (self.y + BULLET_HEIGHT) - (other.y + other.height)
            sx = self.step_x + other.step_x
            sy = self.step_y + other.step_y
            inside = self.is_inside(other)

            self.hit_in = other
            if inside == 0:
                return False

            l = math.fabs(l)
            t = math.fabs(t)
            r = math.fabs(r)
            b = math.fabs(b)

            if (inside == self.LEFT) or (inside == self.RIGHT) or (
                    inside == (self.LEFT | self.TOP) and l > t and self.step_x > 0) or (
                    inside == (self.LEFT | self.BOTTOM) and l > b and self.step_x > 0) or (
                    inside == (self.RIGHT | self.TOP) and r > t and self.step_x < 0) or (
                    inside == (self.RIGHT | self.BOTTOM) and r > b and self.step_x < 0):
                self.step_x *= -1.0
            if inside == self.TOP or inside == self.BOTTOM or (
                    inside == (self.LEFT | self.TOP) and l < t and self.step_y > 0) or (
                    inside == (self.LEFT | self.BOTTOM) and l < b and self.step_y < 0) or (
                    inside == (self.RIGHT | self.TOP) and r < t and self.step_y > 0) or (
                    inside == (self.RIGHT | self.BOTTOM) and r < b and self.step_y < 0):
                self.step_y *= -1.0
            return True
        if self.hit_in == other:
            self.hit_in = None
        return False

    def draw(self, g, image):
        g.draw_image_rect(image, Gempyre.Rect(self.x, self.y, BULLET_WIDTH, BULLET_HEIGHT))


class Gun:
    def __init__(self, mx, my):
        self.mx = mx
        self.my = my
        self.x = self.mx - GUN_WIDTH / 2
        self.y = self.my - GUN_HEIGHT
        self.angle = self.my - GUN_HEIGHT
        self.width = GUN_WIDTH
        self.height = GUN_HEIGHT
        self.step_x = 0
        self.step_y = 0

    def draw(self, g, dome_image, barrel_image):
        g.draw_image_rect(dome_image, Gempyre.Rect(self.mx - GUN_WIDTH / 2, self.my - GUN_HEIGHT, GUN_WIDTH, GUN_HEIGHT))
        g.save()
        g.translate(self.mx, self.my - 20)
        g.rotate(self.angle)
        g.translate(-self.mx, -(self.my - 20))
        g.draw_image_rect(barrel_image, Gempyre.Rect(
            self.mx - BARREL_WIDTH / 2, self.my - (GUN_WIDTH * 0.9), BARREL_WIDTH, BARREL_HEIGHT))
        g.restore()

    def move(self, x, x_min, x_max):
        if (x < 0 and self.x > x_min) or (x > 0 and self.x + GUN_WIDTH < x_max):
            self.step_x = x
            self.mx += x
            self.x += x


class Ammo:
    def __init__(self, width, y_pos):
        self.width = width
        self.y_pos = y_pos
        self.count = MAX_AMMO
        self.max = MAX_AMMO
        self.gap = width / MAX_AMMO - BULLET_WIDTH

    def draw(self, g, image):
        pos = (self.gap + BULLET_WIDTH) * (self.max - self.count)
        for i in range(0, self.count):
            g.draw_image_rect(image, Gempyre.Rect(pos, self.y_pos, BULLET_WIDTH, BULLET_HEIGHT))
            pos += self.gap + BULLET_WIDTH


class Game:
    def __init__(self, ui, canvas, images):
        self.ui = ui
        self.canvas = canvas
        self.barrel = self._get("barrel", images)
        self.barrier = self._get("barrier", images)
        self.bullet = self._get("bullet", images)
        self.dome = self._get("dome", images)
        self.numbers = self._get("numbers", images)
        self.skull = self._get("skull", images)
        self.width = 0
        self.height = 0
        self.bullets = []
        self.monsters = []
        self.rect = None
        self.numberDrawer = None
        self.gun = None
        self.ammo = None
        self.tick = None
        self.hits = 0
        self.game_speed = GAME_SPEED
        self.wave = 0
        self.wave_count = 0
        self.restart = True
        self.on_draw = 0

    @staticmethod
    def _get(name, images):
        for im in images:
            if im[0].startswith(name):
                return im[1]
        return None

    def init(self):
        self.rect = self.canvas.rect()
        self.width = self.rect.width
        self.height = self.rect.height
        self.numberDrawer = Number(self.numbers)
        self.gun = Gun(self.width / 2, self.height - BULLET_HEIGHT - 4)
        self.ammo = Ammo(self.width, self.height - BULLET_HEIGHT - 2)

    def create_monster(self, x_pos):
        self.monsters.append(Monster(
            x_pos, -MONSTER_HEIGHT, MONSTER_WIDTH, MONSTER_HEIGHT, random.randint(1, 99), self.numberDrawer))

    def start(self):
        if self.tick:
            return
        if self.restart:
            self.hits = 0
            self.wave = 0
            self.game_speed = GAME_SPEED
            self.restart = False
        self.ammo.count = MAX_AMMO
        self.wave_count = 3 + self.wave * 1.5
        self.monsters = []
        self.bullets = []
        self.ammo.count = MAX_AMMO
        Gempyre.Element(self.ui, "game_over").set_attribute("style", "visibility:hidden")
        Gempyre.Element(self.ui, "wave_end").set_attribute("style", "visibility:hidden")
        Gempyre.Element(self.ui, "hits").set_html(str(self.hits))
        Gempyre.Element(self.ui, "waves").set_html(str(self.wave + 1))
        Gempyre.Element(self.ui, "monsters").set_html(str(int(self.wave_count + 0.5)))
        Gempyre.Element(self.ui, "instructions").set_attribute("style", "visibility:hidden")
        self.tick = self.ui.start_periodic(timedelta(milliseconds=TICK_SPEED), self.do_tick)
        if self.game_speed > GAME_SPEED / 10:
            self.game_speed -= 1
        self.canvas.draw_completed(lambda: self.draw_completed())
        self.draw_loop()

    def do_tick(self):
        self.game_loop()
        self.on_draw -= 1;
        if self.on_draw > 0:
            return;
        self.on_draw = 10
        self.draw_loop()

    def draw_completed(self):
        self.on_draw = 0

    def game_over(self):
        self.ui.cancel_timer(self.tick);
        self.canvas.draw_completed(None)
        self.tick = None
        Gempyre.Element(self.ui, "game_over").set_attribute("style", "visibility:visible")
        Gempyre.Element(self.ui, "instructions").set_attribute("style", "visibility:visible")
        self.restart = True
        self.draw_loop()

    def wave_end(self):
        self.ui.cancel_timer(self.tick)
        self.canvas.draw_completed(None)
        self.tick = None
        Gempyre.Element(self.ui, "wave_end").set_attribute("style", "visibility:visible")
        self.wave += 1
        self.draw_loop()

    def draw_loop(self):
        fc = Gempyre.FrameComposer()
        fc.clear_rect(Gempyre.Rect(0, 0, self.width, self.height))
        for bullet in self.bullets:
            bullet.draw(fc, self.bullet)
        for monster in self.monsters:
            monster.draw(fc, self.skull)
        self.gun.draw(fc, self.dome, self.barrel)
        self.ammo.draw(fc, self.bullet)
        self.canvas.draw_frame(fc)

    def game_loop(self):
        for bullet in self.bullets:
            bullet.step()
            to_delete = bullet.test_inside(0, 0, self.width, self.height)
            if to_delete:
                self.bullets.remove(bullet)
            else:
                bullet.step()
                bullet.test_hit(self.gun)
                for monster in self.monsters:
                    if bullet.test_hit(monster):
                        monster.endurance -= 1
                        self.hits += 1
                        Gempyre.Element(self.ui, "hits").set_html(str(self.hits))
                        if monster.endurance <= 0:
                            bullet.hit_in = None
                            self.monsters.remove(monster)
                            if len(self.monsters) == 0:
                                self.wave_end()
                                return
                            if self.ammo.count < self.ammo.max:
                                self.ammo.count += 1

                    bullet.step()
        gaps = []
        for monster in self.monsters:
            monster.step()
            if monster.y < 0:
                gaps.append(monster.x)
            to_delete = monster.test_inside(0, 0, self.width, self.height)
            if to_delete:
                self.monsters.remove(monster)
            if (monster.y + monster.height) > (self.height - BULLET_HEIGHT):
                self.game_over()
                return

        if self.wave_count > 0:
            if random.randint(0, self.game_speed) == 1:
                x_pos = random.randint(0, int(self.width - MONSTER_WIDTH))
                is_ok = True
                for x in gaps:
                    if (x_pos > x) or (x_pos < x + MONSTER_WIDTH):
                        is_ok = False
                        break
                if is_ok:
                    self.create_monster(x_pos)
                    self.wave_count -= 1

    def shoot(self):
        if not self.tick:
            return

        if self.ammo.count > 0:
            start_x = (self.gun.mx - 10) + 80 * math.sin(self.gun.angle)
            start_y = (self.height - 40 - 10) - 80 * math.cos(self.gun.angle)
            self.bullets.append(Bullet(math.pi - self.gun.angle, start_x, start_y))
            self.ammo.count -= 1

    def turret(self, angle):
        if angle > -math.pi and angle - math.pi:
            self.gun.angle = angle
        self.draw_loop()

    def turret_turn(self, angle):
        self.turret(self.gun.angle + angle)

    def gun_move(self, x):
        self.gun.move(x, 0, self.width)

def main():
    root = os.path.dirname(sys.argv[0]) + '/assets/'
    files = ["balls.html", "barrel.png", "barrier.png", "bullet.png", "dome.png", "numbers.png", "skull.png"]
    full_paths = list(map(lambda f: root + f, files))
    data_map, names = resource.from_file_list(full_paths)

    # make list of image URIs from names
    urls = []
    for i in range(1, len(full_paths)):
        urls.append(names[full_paths[i]])

    ui = Gempyre.Ui(data_map, names[full_paths[0]], "Balls", 720, 920)

    Gempyre.Element(ui, "game_over").set_attribute("style", "visibility:hidden")
    Gempyre.Element(ui, "wave_end").set_attribute("style", "visibility:hidden")

    canvas = Gempyre.CanvasElement(ui, 'canvas')

    images = [canvas.add_image(url, None) for url in urls]
    game = Game(ui, canvas, zip(files[1:], images))

    ui.on_open(lambda: game.init())

    canvas.subscribe("click", lambda _: game.shoot(), [], timedelta(milliseconds=200))

    def get_property(event):
        if event:
            nonlocal game
            x = float(event.properties["clientX"])
            y = float(event.properties["clientY"])
            mid_x = game.gun.x
            return math.atan2((game.rect.height - y), mid_x - x) - math.pi / 2
        return 0

    canvas.subscribe('mousemove', lambda e: game.turret(get_property(e)),
                     ["clientX", "clientY"], timedelta(milliseconds=100))

    def key_listen(e):
        code = int(float((e.properties['keyCode'])))
        if code == 37:  # left arrow
            game.gun_move(-GUN_SPEED)
        elif code == 39:  # right arrow
            game.gun_move(GUN_SPEED)
        elif code == ord('Z'):
            game.turret_turn(-TURRET_STEP)
        elif code == ord('X'):
            game.turret_turn(TURRET_STEP)
        elif code == ord('C'):
            game.shoot()
        elif code == ord('A'):
            game.start()

    # canvas is not focusable therefore we listen whole app
    ui.root().subscribe('keydown', key_listen, ['keyCode'])
    ui.run()

if __name__ == "__main__":
    #Gempyre.set_debug()
    main()



import Gempyre
from Gempyre import resource
import random
import math
from datetime import timedelta  # for time periods
import os
import sys


def normalize(r):
    while r < 0:
        r += 2 * math.pi
    while r >= 2 * math.pi:
        r -= 2 * math.pi
    return r


def _towards(r0, r, step):
    delta = r0 - r
    if delta < 0:
        if delta < math.pi:
            return r0 + step
        elif delta > math.pi:
            return r0 - step
        else:
            return r0 + step * random.randint(-1, 1)
    elif delta > 0:
        if delta > math.pi:
            return r0 + step
        elif delta < math.pi:
            return r0 - step
        else:
            return r0 + step * random.randint(-1, 1)


def towards(r0, r, step):
    return normalize(_towards(r0, r, step))


class Bee:
    width = 10
    height = 10
    min_distance = 300
    sight_len = 5

    def __init__(self, swarm):
        self.x = 0
        self.y = 0
        self.direction = random.random() * math.pi * 2
        self.heading = self.direction
        self.swarm = swarm
        self.speed = 1
        self.chasing = False
        self.see_x = 0
        self.see_y = 0

    def move(self):
        bee, dist = self.swarm.closest(self)
        if not bee:
            return
        if dist < self.min_distance:
            xx = self.x - bee.x
            yy = self.y - bee.y
            self.direction = math.atan2(yy, xx)
            self.chasing = False
        else:
            xx = self.x - self.swarm.fake_bee.x
            yy = self.y - self.swarm.fake_bee.y
            self.direction = math.atan2(yy, xx) + math.pi
            self.chasing = True
        self.heading = towards(self.heading, self.direction, 0.03)
        next_x = math.cos(self.heading) * self.speed
        next_y = math.sin(self.heading) * self.speed
        self.x += next_x
        self.y += next_y

        self.see_x = self.x + next_x * self.sight_len
        self.see_y = self.y + next_y * self.sight_len

    def draw(self, drawer):
        d = self.heading
        drawer.save()
        if self.chasing:
            drawer.fill_style('black')
        else:
            drawer.fill_style('blue')
        drawer.translate(self.x, self.y)
        drawer.rotate(d)
        drawer.translate(-self.x, -self.y)
        drawer.begin_path()
        drawer.move_to(self.x - self.width / 2, self.y - self.height / 2)
        drawer.line_to(self.x + self.width / 2, self.y)
        drawer.line_to(self.x - self.width / 2, self.y + self.height / 2)
        drawer.line_to(self.x - self.width / 4, self.y)
        drawer.fill()
        drawer.restore()


class Swarm:
    def __init__(self, bee_count):
        self.x = 0
        self.y = 0
        self.width = 0
        self.height = 0
        self.bees = []
        self.fake_bee = Bee(self)
        for i in range(0, bee_count):
            self.bees.append(Bee(self))

    def closest(self, bee):
        closest_one = None
        closest_one_dist = self.width * self.height
        for b in self.bees:
            if b != bee:
                dx = bee.see_x - b.see_x
                dy = bee.see_y - b.see_y
                dist = dx * dx + dy * dy
                if dist < closest_one_dist:
                    closest_one_dist = dist
                    closest_one = b
        return closest_one, closest_one_dist

    def mid_point(self):
        return (self.x + self.width / 2,
                self.y + self.height / 2)

    def set_pos(self, x, y):
        self.fake_bee.x = x
        self.fake_bee.y = y

    def move(self):
        for b in self.bees:
            b.move()

    def draw(self, drawer):
        for b in self.bees:
            b.draw(drawer)
        drawer.fill_rect(Gempyre.Rect(self.fake_bee.x - 5, self.fake_bee.y - 5, 10, 10))


def main():
    print(Gempyre.version())
    #Gempyre.set_debug()
    current_dir = os.path.dirname(sys.argv[0])
    file_map, names = resource.from_file(current_dir + "/swarm.html")
    ui = Gempyre.Ui(file_map, '/swarm.html', "Swarm", 800 + 15, 600 + 20)
    canvas = Gempyre.CanvasElement(ui, "canvas")
    swarm = Swarm(200)
    canvas_rect = Gempyre.Rect()

    def resize_handler(_):
        nonlocal canvas_rect
        canvas_rect = canvas.rect()
        swarm.x = 0
        swarm.y = 0
        swarm.width = canvas_rect.width
        swarm.height = canvas_rect.height

    ui.root().subscribe("resize", resize_handler)

    def draw():
        frame = Gempyre.FrameComposer()
        frame.clear_rect(Gempyre.Rect(swarm.x, swarm.y, swarm.width, swarm.height))
        swarm.draw(frame)
        frame.fill_rect(Gempyre.Rect(swarm.fake_bee.x - 5, swarm.fake_bee.y - 5, 10, 10))
        canvas.draw_frame(frame)

    def on_start():
        resize_handler(None)
        for b in swarm.bees:
            b.x = random.randint(b.width + swarm.x, swarm.width)
            b.y = random.randint(b.height + swarm.y, swarm.height)
        draw()

    ui.on_open(on_start)

    ui.start_periodic(timedelta(milliseconds=40), lambda: swarm.move())

    canvas.draw_completed(draw)

    def make_move(event):
        mouse_x = float(event.properties['clientX']) - canvas_rect.x
        mouse_y = float(event.properties['clientY']) - canvas_rect.y
        swarm.set_pos(mouse_x, mouse_y)

    canvas.subscribe('mousemove', make_move,
                     ["clientX", "clientY"], timedelta(milliseconds=100))

    ui.run()


if __name__ == "__main__":
    main()



import sys
import os
import subprocess
import platform
import shlex
import psutil

import Gempyre
from Gempyre import resource

html = '''<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <title>test</title>
</head>
<body>
  <script src="/gempyre.js"></script>
  <table id="info">
  <tr id="header">
  </tr>
  </table>
  <input id="command_line" type="text"></input>
  <button id="run">Run</button>
  <pre id="output"></pre>
</body>
</html>
'''

map, names = resource.from_bytes({"main.html": bytes(html, 'utf-8')})

Gempyre.set_debug()

ui = Gempyre.Ui(map, names["main.html"])

header = Gempyre.Element(ui, "header");
Gempyre.Element(ui, "th", header).set_html("Name")
Gempyre.Element(ui, "th", header).set_html("Info")

name = list()

name.append(["Architecture", platform.architecture()[0]])
name.append(["Machine", platform.machine()])
name.append(["Node", platform.node()])
name.append(["System", platform.system()])

info = Gempyre.Element(ui, "info")
for n in name:
    h = Gempyre.Element(ui, "tr", info)
    Gempyre.Element(ui, "td", h).set_html(n[0])
    Gempyre.Element(ui, "td", h).set_html(n[1])
    
input = Gempyre.Element(ui, "command_line")
output = Gempyre.Element(ui, "output")

process = psutil.Process(os.getpid())
parent = process.parent()
decoding = 'CP850' if parent and parent.name() == 'cmd.exe' else 'utf-8'


def do_run(ev):
    global input
    global output
    value = input.values()
    if not value or len(value['value']) == 0:
        return
    command_line = shlex.split(value['value'])
    
    try:
        out = subprocess.run(command_line, stdout=subprocess.PIPE, shell=True).stdout.decode(decoding)
    except FileNotFoundError as e:
         output.set_html(str(e))
    else:
        output.set_html(out)   
               

Gempyre.Element(ui, "run").subscribe('click', do_run)    
    
ui.run()





import sys
import base64
import re
import os
from typing import Dict

def from_file_list(lst):
    """
    Generates a filemap for Gempyre
    :param lst: list of filenames.
    :return: two dictionaries, first is map second is to map it's keys to give arguments.
    """
    data = dict()
    names = dict()
    for in_name in lst:
        encoded = ""
        with open(in_name, 'rb') as infile:
            content = infile.read()
            encoded = base64.standard_b64encode(content).decode('utf-8')
        sname = '/' + re.sub('[^a-zA-Z_.]', '', os.path.basename(in_name)) # why this was capitalize()?
        data[sname] = encoded
        names[in_name] = sname
    return data, names
    
    
def from_file(*argv):
    """
    Generates a filemap for Gempyre
    :param argv: argument list of filenames.
    :return: two dictionaries, first is map second is to map it's keys to give arguments.
    """
    lst = list()
    for a in argv:
        lst.append(a);
    return from_file_list(lst)

def from_bytes(data_map: Dict[str, bytes]):
    """
    Generates a filemap for Gempyre
    :param dict of file (kind of) names and their content.
    :return: two dictionaries, first is map second is to map it's keys to give arguments.
    """
    data = dict()
    names = dict()
    for in_name, content in data_map.items():
        encoded = base64.standard_b64encode(content).decode('utf-8')
        sname = '/' + re.sub('[^a-zA-Z_.]', '', os.path.basename(in_name)) # why this was capitalize()?
        data[sname] = encoded
        names[in_name] = sname
    return data, names

def from_html(html: str):
    return from_bytes({"main.html": bytes(html, 'utf-8')})


import sys
import os
import math
import Gempyre
from datetime import datetime
from datetime import date
from datetime import timedelta
from Gempyre import resource

name = sys.argv[1] if len(sys.argv) > 1 else os.path.join(os.path.dirname(sys.argv[0]), "python_test_1.html")

map, names = resource.from_file(name)
print(names[name], name)

ui = Gempyre.Ui(map, names[name], "Test app") if sys.platform != 'win32' or sys.version_info < (
    3, 8) else Gempyre.Ui(map, names[name], "")

ver, major, minor = Gempyre.version()
Gempyre.Element(ui, 'ver').set_html("Gempyre Version: " + str(ver) + '.' + str(major) + '.' + str(minor));

eyes = Gempyre.CanvasElement(ui, 'eyes')
eyes_rect = None


def on_start():
    global eyes_rect
    eyes_rect = eyes.rect()
    print(ui.root().html())
    ui.start_periodic(timedelta(seconds=1), lambda: Gempyre.Element(ui,'time').set_html(
                       datetime.now().strftime("%d/%m/%Y %H:%M:%S")))


ui.on_open(on_start)

elementCount = 0

addbutton = Gempyre.Element(ui, 'do')


def add_element(event):
    global elementCount
    new = Gempyre.Element(ui, "name_" + str(elementCount), "img", ui.root())
    elementCount += 1
    new.set_attribute("SRC", "https://www.animatedimages.org/data/media/202/animated-dog-image-0931.gif")

    
addbutton.subscribe('click', add_element)

removebutton = Gempyre.Element(ui, 'take')


def removeElement(event):
    global elementCount
    if elementCount <= 0:
        return
    elementCount -= 1
    Gempyre.Element(ui, "name_" + str(elementCount)).remove()


removebutton.subscribe('click', removeElement)

ui.on_exit(lambda: print("on Exit"))

tick1 = Gempyre.Element(ui, 'tick1')
tick2 = Gempyre.Element(ui, 'tick2')


def toggle(e0, e1):
    if e0.values()['checked'] == 'true':
        e1.set_attribute('checked', 'false')
    else:
        e1.set_attribute('checked', 'true')


tick1.subscribe('click', lambda _: toggle(tick1, tick2))
tick2.subscribe('click', lambda _: toggle(tick2, tick1))
tick1.set_attribute('checked', 'true')

def move_eyes(x, y):
    fc = Gempyre.FrameComposer()
    fc.clear_rect(Gempyre.Rect(0, 0, 80, 80))
    
    def draw_eye(fc, px, py, x, y):
        fc.begin_path()
        fc.arc(px, py, 20, 0, 2 * math.pi)
        fc.stroke_style('black')
        fc.stroke()
    
        angle = math.atan2(x, y)
        dx = math.fabs(x - px)
        dy = math.fabs(y - py)
        xx = math.sin(angle) * min(dx, 10)
        yy = math.cos(angle) * min(dy, 10)
        fc.begin_path()
        fc.arc(px + xx, py + yy, 10, 0, 2 * math.pi)
        fc.fill_style('black')
        fc.fill()
    
    draw_eye(fc, 20, 40, x, y)
    draw_eye(fc, 60, 40, x, y)
    eyes.draw_frame(fc)
    
ui.root().subscribe('mousemove', lambda e: move_eyes(float(e.properties['clientX']) - eyes_rect.x,
                                                     float(e.properties['clientY']) - eyes_rect.y),
                     ["clientX", "clientY"], timedelta(milliseconds=100))

kitt_canvas = Gempyre.CanvasElement(ui, "kitt")
g = Gempyre.Bitmap(200, 20)
position = 1
direction = 1


def draw_kitt():
    global position
    global direction
    position += direction
    if position >= 198:
        direction = -1
    if position <= 0:
        direction = 1;
    g.draw_rect(Gempyre.Rect(0, 0, 200, 20), Gempyre.Black)
    for i in range(0, 20):
        g.set_pixel(position - 1, i, Gempyre.Red)
    for i in range(0, 20):
        g.set_pixel(position, i, Gempyre.Bitmap.pix(0xFF, 0xFF, 0))
    for i in range(0, 20):
        g.set_pixel(position + 1, i, Gempyre.Red)

# set redraw
kitt_canvas.draw_completed(lambda: kitt_canvas.draw_bitmap(g))
# set graphics motion update, i.e. fps and anitmation are now decoupled (which is important)
ui.start_periodic(timedelta(milliseconds=10), draw_kitt)
# draw 1st time, draw_completed does redraw
kitt_canvas.draw_bitmap(g)

ui.run()





import Gempyre
import inspect
import urllib.request
import re

DOX_FUNC="https://mmertama.github.io/Gempyre/functions_func.html"


def list_items(module, context):
    content = {'function':[], 'class':[], 'method':[], 'data':[]}
    for name, obj in inspect.getmembers(module):
        if name.startswith('__') and name.endswith('__'):
            continue
        if inspect.isclass(obj):
            content['class'].append(f"{context}::{name}")
            for k, v in list_items(obj, f"{context}::{name}").items():
                if len(v) > 0:
                    content[k].extend(v)
        elif inspect.isfunction(obj):
            content['function'].append(f"{context}::{name}")
        elif inspect.ismethod(obj):
            content['method'].append(f"{context}::{name}")
        elif inspect.isroutine(obj):
            content['method'].append(f"{context}::{name}")    
        elif inspect.isdatadescriptor(obj): # Properties, etc.
            content['data'].append(f"{context}::{name}")                  
    return content

def main():
    items = list_items(Gempyre, 'Gempyre')
    with urllib.request.urlopen(DOX_FUNC) as fp:
        bytes = fp.read()
        html = bytes.decode("utf8")
    
    functions = []    
    for line in html.split('\n'): 
        m = re.match(r'\s*<li>([a-zA-Z_][a-zA-Z0-9_]*)\(\).*\>([a-zA-Z_:][a-zA-Z0-9_:]*)</a></li>', line)
        if m:
            els = m[2].split('::')
            els.append(m[1]) 
            functions.append(els)
            
    exceptions = set()
    exceptions_map = {}
    with open('exceptions.txt') as f:
        for line in f.readlines():
            m = re.match(r'\s*([a-zA-Z_][^# ]+)', line)
            if m:
                exceptions.add(m[1])
            m = re.match(r'\s*([a-zA-Z_]\s*->\s*([a-zA-Z_])[^# ]+)', line)
            if m:
                exceptions_map[m[1]] = m[2]                    
    
    for func in functions:
        name = '::'.join(func)
        if name not in items['method'] and name not in exceptions and (
            name not in exceptions_map or exceptions_map[name] not in items['method']) and func[-1] != func[-2]: # skip constructors
            print(f"'{name}'")
        

if __name__ == "__main__":
    main()


from setuptools import find_packages
from skbuild import setup  # This line replaces 'from setuptools import setup'
import os

setup (name='Gempyre',
       packages=["Gempyre", "Gempyre_utils"],
       package_dir={"": "src"},
       cmake_install_dir="src/Gempyre",
      )




from setuptools import setup, Extension, find_packages
import os
import sys
import platform
import subprocess


setup (
        name='Gempyre',
        version='1.6.1',
        license='MIT',
        description='Gempyre Framework',
        author='Markus Mertama',
        author_email='foobar@foobar',
        url='https://github.com/mmertama',
        long_description='''
Gempyre is C++ Framework for quick and simple UI development and Gempyre-Python apply that breeze to Python development.
''',
        packages=find_packages(),
        setup_requires=['wheel'],
        install_requires=['pywebview', 'websockets',],
        entry_points={
           'console_scripts': ['pyclient=client.pyclient:main']},
        package_data={
            'Gempyre': ['macos_lib/*', 'unix_lib/*', 'windows_lib/*']},
        classifiers=[
            'License :: OSI Approved :: MIT License',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
        ],
        python_requires='>=3.8',   
      )
       
